# This module is imported by nwbwidgets when
# nwbwidgets.load_extension_widgets_into_spec([ndx_ophys_devices])
# is called. Otherwise, the module is not imported unless explicitly imported.

from .tetrode_series_widget import TetrodeSeriesWidget
from .. import TetrodeSeries

vis_spec = {
    TetrodeSeries: TetrodeSeriesWidget,
}



"""Test in-memory Python API constructors for ndx-ophys-devices extension."""

import pytest

from ndx_ophys_devices.testing import (
    mock_DeviceModel,
    mock_Indicator,
    mock_Effector,
    mock_OpticalFiber,
    mock_Photodetector,
    mock_DichroicMirror,
    mock_OpticalFilter,
    mock_BandOpticalFilter,
    mock_EdgeOpticalFilter,
    mock_ObjectiveLens,
    mock_ExcitationSource,
    mock_PulsedExcitationSource,
)


def test_constructor_device_model():
    mock_DeviceModel()


def test_constructor_indicator():
    mock_Indicator()


def test_constructor_effector():
    mock_Effector()


def test_constructor_optical_fiber():
    mock_OpticalFiber()


def test_constructor_photodetector():
    mock_Photodetector()


def test_constructor_dichroic_mirror():
    mock_DichroicMirror()


def test_constructor_optical_filter():
    mock_OpticalFilter()


def test_constructor_band_optical_filter():
    mock_BandOpticalFilter()


def test_constructor_edge_optical_filter():
    mock_EdgeOpticalFilter()


def test_constructor_objective_lens():
    mock_ObjectiveLens()


def test_constructor_excitation_source():
    mock_ExcitationSource()


def test_constructor_pulsed_excitation_source():
    mock_PulsedExcitationSource()


if __name__ == "__main__":
    pytest.main()  # Required since not a typical package structure



from typing import Optional

from pynwb.testing.mock.utils import name_generator

import ndx_ophys_devices


def mock_DeviceModel(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a DeviceModel type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock DeviceModel.",
    model: str = "A fake model of the mock DeviceModel.",
) -> ndx_ophys_devices.DeviceModel:
    device_model = ndx_ophys_devices.DeviceModel(
        name=name or name_generator("DeviceModel"),
        description=description,
        manufacturer=manufacturer,
        model=model,
    )
    return device_model


def mock_Indicator(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a Indicator type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock indicator.",
    label: str = "A fake label of the indicator.",
    injection_brain_region: str = "A fake injection brain region of the indicator.",
    injection_coordinates_in_mm: list = [3.0, 2.0, 1.0],
) -> ndx_ophys_devices.Indicator:
    indicator = ndx_ophys_devices.Indicator(
        name=name or name_generator("Indicator"),
        description=description,
        manufacturer=manufacturer,
        label=label,
        injection_brain_region=injection_brain_region,
        injection_coordinates_in_mm=injection_coordinates_in_mm,
    )
    return indicator


def mock_Effector(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a Effector type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock effector.",
    label: str = "A fake label of the effector.",
    injection_brain_region: str = "A fake injection brain region of the effector.",
    injection_coordinates_in_mm: list = [3.0, 2.0, 1.0],
) -> ndx_ophys_devices.Effector:
    effector = ndx_ophys_devices.Effector(
        name=name or name_generator("Effector"),
        description=description,
        manufacturer=manufacturer,
        label=label,
        injection_brain_region=injection_brain_region,
        injection_coordinates_in_mm=injection_coordinates_in_mm,
    )
    return effector


def mock_OpticalFiber(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a OpticalFiber type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an optical fiber.",
    model: str = "A fake model of the mock an optical fiber.",
    numerical_aperture: float = 0.2,
    core_diameter_in_um: float = 400.0,
) -> ndx_ophys_devices.OpticalFiber:
    optical_fiber = ndx_ophys_devices.OpticalFiber(
        name=name or name_generator("OpticalFiber"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        numerical_aperture=numerical_aperture,
        core_diameter_in_um=core_diameter_in_um,
    )
    return optical_fiber


def mock_Photodetector(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a Photodetector type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an photodetector.",
    model: str = "A fake model of the mock an photodetector.",
    detector_type: str = "PMT",
    detected_wavelength_in_nm: float = 520.0,
    gain: float = 100.0,
    gain_unit: str = "A/W",
) -> ndx_ophys_devices.Photodetector:
    photodetector = ndx_ophys_devices.Photodetector(
        name=name or name_generator("Photodetector"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        detector_type=detector_type,
        detected_wavelength_in_nm=detected_wavelength_in_nm,
        gain=gain,
        gain_unit=gain_unit,
    )
    return photodetector


def mock_DichroicMirror(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a DichroicMirror type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an dichroic mirror.",
    model: str = "A fake model of the mock an dichroic mirror.",
    cut_on_wavelength_in_nm: float = 470.0,
    cut_off_wavelength_in_nm: float = 500.0,
    reflection_band_in_nm: list = [460.0, 480.0],
    transmission_band_in_nm: list = [490.0, 520.0],
    angle_of_incidence_in_degrees: float = 45.0,
) -> ndx_ophys_devices.DichroicMirror:
    dichroic_mirror = ndx_ophys_devices.DichroicMirror(
        name=name or name_generator("DichroicMirror"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        cut_on_wavelength_in_nm=cut_on_wavelength_in_nm,
        cut_off_wavelength_in_nm=cut_off_wavelength_in_nm,
        reflection_band_in_nm=reflection_band_in_nm,
        transmission_band_in_nm=transmission_band_in_nm,
        angle_of_incidence_in_degrees=angle_of_incidence_in_degrees,
    )
    return dichroic_mirror


def mock_OpticalFilter(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a OpticalFilter type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an optical filter.",
    model: str = "A fake model of the mock an optical filter.",
    filter_type: str = "Longpass",
) -> ndx_ophys_devices.EdgeOpticalFilter:
    optical_filter = ndx_ophys_devices.OpticalFilter(
        name=name or name_generator("OpticalFilter"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        filter_type=filter_type,
    )
    return optical_filter


def mock_BandOpticalFilter(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a BandOpticalFilter type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an band optical filter.",
    model: str = "A fake model of the mock an band optical filter.",
    center_wavelength_in_nm: float = 505.0,
    bandwidth_in_nm: float = 30.0,  # 505±15nm
    filter_type: str = "Bandpass",
) -> ndx_ophys_devices.BandOpticalFilter:
    band_optical_filter = ndx_ophys_devices.BandOpticalFilter(
        name=name or name_generator("BandOpticalFilter"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        center_wavelength_in_nm=center_wavelength_in_nm,
        bandwidth_in_nm=bandwidth_in_nm,
        filter_type=filter_type,
    )
    return band_optical_filter


def mock_EdgeOpticalFilter(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a EdgeOpticalFilter type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an edge optical filter.",
    model: str = "A fake model of the mock an edge optical filter.",
    cut_wavelength_in_nm: float = 585.0,
    slope_in_percent_cut_wavelength: float = 1.0,
    slope_starting_transmission_in_percent: float = 10.0,
    slope_ending_transmission_in_percent: float = 80.0,
    filter_type: str = "Longpass",
) -> ndx_ophys_devices.EdgeOpticalFilter:
    edge_optical_filter = ndx_ophys_devices.EdgeOpticalFilter(
        name=name or name_generator("EdgeOpticalFilter"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        cut_wavelength_in_nm=cut_wavelength_in_nm,
        slope_in_percent_cut_wavelength=slope_in_percent_cut_wavelength,
        slope_starting_transmission_in_percent=slope_starting_transmission_in_percent,
        slope_ending_transmission_in_percent=slope_ending_transmission_in_percent,
        filter_type=filter_type,
    )
    return edge_optical_filter


def mock_ObjectiveLens(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a ObjectiveLens type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock an objective lens.",
    model: str = "A fake model of the mock an objective lens.",
    numerical_aperture: float = 0.2,
    magnification: float = 10.0,
) -> ndx_ophys_devices.ObjectiveLens:
    objective_lens = ndx_ophys_devices.ObjectiveLens(
        name=name or name_generator("ObjectiveLens"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        numerical_aperture=numerical_aperture,
        magnification=magnification,
    )
    return objective_lens


def mock_ExcitationSource(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a ExcitationSource type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock excitation source.",
    model: str = "A fake model of the mock excitation source.",
    illumination_type: str = "Laser.",
    excitation_wavelength_in_nm: float = 500.0,
    power_in_W: float = 0.7,
    intensity_in_W_per_m2: float = 0.005,
    exposure_time_in_s: float = 2.51e-13,
) -> ndx_ophys_devices.ExcitationSource:
    excitation_source = ndx_ophys_devices.ExcitationSource(
        name=name or name_generator("ExcitationSource"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        illumination_type=illumination_type,
        excitation_wavelength_in_nm=excitation_wavelength_in_nm,
        power_in_W=power_in_W,
        intensity_in_W_per_m2=intensity_in_W_per_m2,
        exposure_time_in_s=exposure_time_in_s,
    )
    return excitation_source


def mock_PulsedExcitationSource(
    *,
    name: Optional[str] = None,
    description: str = "This is a mock instance of a PulsedExcitationSource type to be used for rapid testing.",
    manufacturer: str = "A fake manufacturer of the mock excitation source.",
    model: str = "A fake model of the mock excitation source.",
    illumination_type: str = "Laser.",
    excitation_wavelength_in_nm: float = 500.0,
    peak_power_in_W: float = 0.7,
    peak_pulse_energy_in_J: float = 0.7,
    intensity_in_W_per_m2: float = 0.005,
    exposure_time_in_s: float = 2.51e-13,
    pulse_rate_in_Hz: float = 2.0e6,
) -> ndx_ophys_devices.PulsedExcitationSource:
    pulsed_excitation_source = ndx_ophys_devices.PulsedExcitationSource(
        name=name or name_generator("PulsedExcitationSource"),
        description=description,
        manufacturer=manufacturer,
        model=model,
        illumination_type=illumination_type,
        excitation_wavelength_in_nm=excitation_wavelength_in_nm,
        peak_power_in_W=peak_power_in_W,
        peak_pulse_energy_in_J=peak_pulse_energy_in_J,
        intensity_in_W_per_m2=intensity_in_W_per_m2,
        exposure_time_in_s=exposure_time_in_s,
        pulse_rate_in_Hz=pulse_rate_in_Hz,
    )
    return pulsed_excitation_source


from ._mock import (
    mock_DeviceModel,
    mock_Indicator,
    mock_Effector,
    mock_OpticalFiber,
    mock_Photodetector,
    mock_DichroicMirror,
    mock_OpticalFilter,
    mock_BandOpticalFilter,
    mock_EdgeOpticalFilter,
    mock_ObjectiveLens,
    mock_ExcitationSource,
    mock_PulsedExcitationSource,
)

__all__ = [
    "mock_DeviceModel",
    "mock_Indicator",
    "mock_Effector",
    "mock_OpticalFiber",
    "mock_Photodetector",
    "mock_DichroicMirror",
    "mock_OpticalFilter",
    "mock_BandOpticalFilter",
    "mock_EdgeOpticalFilter",
    "mock_ObjectiveLens",
    "mock_ExcitationSource",
    "mock_PulsedExcitationSource",
]



# If NWBWidgets is installed, create a custom widget for the TetrodeSeries
# neurodata type.
#
# You will also need to update the `vis_spec` dictionary in `__init__.py` so
# that nwbwidgets can find your custom visualizations.
#
# Example usage:
#   from nwbwidgets import nwb2widget, load_extension_widgets_into_spec
#   load_extension_widgets_into_spec([ndx_ophys_devices])
#   nwb2widget(nwbfile)

try:
    from nwbwidgets.ecephys import ElectricalSeriesWidget
    from ipywidgets import widgets

    from .. import TetrodeSeries

    # TODO define your own custom widget for your extension neurodata type
    # using TetrodeSeriesWidget as an example.
    class TetrodeSeriesWidget(ElectricalSeriesWidget):  # this is an HBox
        """Show the trode_id above the ElectricalSeries widget"""

        def __init__(self, tetrode_series: TetrodeSeries, **kwargs):
            super().__init__(electrical_series=tetrode_series, **kwargs)
            vbox = widgets.VBox(
                children=[
                    self._create_trode_id_box(tetrode_series),
                    widgets.HBox(children=list(self.children)),
                ]
            )
            self.children = [vbox]

        def _create_trode_id_box(self, tetrode_series: TetrodeSeries):
            field_lay = widgets.Layout(
                max_height="40px",
                max_width="600px",
                min_height="30px",
                min_width="130px",
            )
            key = widgets.Label("trode_id:", layout=field_lay)
            val = widgets.Label(str(tetrode_series.trode_id), layout=field_lay)
            return widgets.HBox(children=[key, val])

    # add the widget class to the TetrodeSeries class
    TetrodeSeries.widget = TetrodeSeriesWidget

except ImportError:
    print("NWBWidgets is not installed, so we cannot create a new widget.")  # noqa: T201
    print("Run `pip install nwbwidgets` to install NWBWidgets.")  # noqa: T201



import os
from pynwb import load_namespaces, get_class

try:
    from importlib.resources import files
except ImportError:
    # TODO: Remove when python 3.9 becomes the new minimum
    from importlib_resources import files

# Get path to the namespace.yaml file with the expected location when installed not in editable mode
__location_of_this_file = files(__name__)
__spec_path = __location_of_this_file / "spec" / "ndx-ophys-devices.namespace.yaml"

# If that path does not exist, we are likely running in editable mode. Use the local path instead
if not os.path.exists(__spec_path):
    __spec_path = __location_of_this_file.parent.parent.parent / "spec" / "ndx-ophys-devices.namespace.yaml"

# Load the namespace
load_namespaces(str(__spec_path))


DeviceModel = get_class("DeviceModel", "ndx-ophys-devices")
Indicator = get_class("Indicator", "ndx-ophys-devices")
OpticalFiber = get_class("OpticalFiber", "ndx-ophys-devices")
ExcitationSource = get_class("ExcitationSource", "ndx-ophys-devices")
PulsedExcitationSource = get_class("PulsedExcitationSource", "ndx-ophys-devices")
Photodetector = get_class("Photodetector", "ndx-ophys-devices")
DichroicMirror = get_class("DichroicMirror", "ndx-ophys-devices")
OpticalFilter = get_class("OpticalFilter", "ndx-ophys-devices")
BandOpticalFilter = get_class("BandOpticalFilter", "ndx-ophys-devices")
EdgeOpticalFilter = get_class("EdgeOpticalFilter", "ndx-ophys-devices")
ObjectiveLens = get_class("ObjectiveLens", "ndx-ophys-devices")
Effector = get_class("Effector", "ndx-ophys-devices")

del load_namespaces, get_class
